@@ -2,4 +2,5 @@ from django.apps import AppConfig
 
 
 class AuthConfig(AppConfig):
-    name = 'auth0'
+    name = 'st_common_data.auth'
+    label = 'auth0'
